//
//  EDNViewController.m
//  BasemapSample
//
//  Created by Nicholas Furness on 7/11/12.
//  Copyright (c) 2012 ESRI. All rights reserved.
//

#import <ArcGIS/ArcGIS.h>
#import "EDNViewController.h"

#define kTopoWebMapID @"6e03e8c26aad4b9c92a87c1063ddb0e3"
#define kStreetWebMapID @"d8855ee4d3d74413babfb0f41203b168"
#define kGreyWebMapID @"8b3b470883a744aeb60e5fff0a319ce7"
#define kImageryWebMapID @"86de95d4e0244cba80f0fa2c9403a7b2"

#define kTopoIndex 0
#define kStreetIndex 1
#define kGreyIndex 2
#define kImageryIndex 3

@interface EDNViewController () <AGSWebMapDelegate>
// The Map View
@property (weak, nonatomic) IBOutlet AGSMapView *mapView;

// Load and hold on to webmaps we'll use as basemaps.
@property (nonatomic, strong) AGSWebMap *topoWebMap;
@property (nonatomic, strong) AGSWebMap *streetWebMap;
@property (nonatomic, strong) AGSWebMap *imageryWebMap;
@property (nonatomic, strong) AGSWebMap *greyWebMap;

// The webmap with the actual content we want to use
@property (nonatomic, strong) AGSWebMap *mainWebMapWithContents;

// Track whether we've successfully connected with the various web maps
// we're going to use as basemaps.
@property (nonatomic, strong) NSMutableDictionary *loadedStates;

// When we switch basemap, we'll need to re-set the extent after 
// the webmap has re-loaded or else it'll just go back to its default extent.
// This property is used to track it.
@property (nonatomic, strong) AGSEnvelope *mapExtentAtBasemapSwitch;

// The handler for when the user picks a new basemap.
- (IBAction)basemapTypeChanged:(id)sender;
@end

@implementation EDNViewController
@synthesize mapView = _mapView;
@synthesize topoWebMap = _topoWebMap;
@synthesize streetWebMap = _streetWebMap;
@synthesize imageryWebMap = _aerialWebMap;
@synthesize greyWebMap = _greyWebMap;
@synthesize mainWebMapWithContents = _mainWebMapWithContents;
@synthesize loadedStates = _loadedStates;
@synthesize mapExtentAtBasemapSwitch = _mapExtentAtBasemapSwitch;

#pragma mark - Load up the webmaps for content and basemaps...

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    self.loadedStates = [NSMutableDictionary dictionary];
    
    // Get hold of the webmaps that we'll use as basemaps.
    self.topoWebMap = [AGSWebMap webMapWithItemId:kTopoWebMapID credential:nil];
    self.topoWebMap.delegate = self;
    self.streetWebMap = [AGSWebMap webMapWithItemId:kStreetWebMapID credential:nil];
    self.streetWebMap.delegate = self;
    self.imageryWebMap = [AGSWebMap webMapWithItemId:kImageryWebMapID credential:nil];
    self.imageryWebMap.delegate = self;
    self.greyWebMap = [AGSWebMap webMapWithItemId:kGreyWebMapID credential:nil];
    self.greyWebMap.delegate = self;

    // The WebMap ID for the content is: fca4b4f0cb1346f2a3a906abe7dcfaa5
    // Any WebMap ID should do...
    // 78acf73ae5df4a569231229a636dcc72 <- same content, different map
    // 8a567ebac15748d39a747649a2e86cf4 <- London Underground
    // 0bb40265b64a4b29ac3bccb774413357 <- World wetlands.
    // Load the webmap with our contents.
    self.mainWebMapWithContents = [AGSWebMap webMapWithItemId:@"fca4b4f0cb1346f2a3a906abe7dcfaa5" 
                                                   credential:nil];
    self.mainWebMapWithContents.delegate = self;
    [self.mainWebMapWithContents openIntoMapView:self.mapView];
}

#pragma mark - Web Map Delegate Handlers

// When a webmap loads, this is called (since we set the delegates above)
- (void) webMapDidLoad:(AGSWebMap *)webMap
{
    // When each webmap loads, let's know about it.
    if (webMap == self.mainWebMapWithContents)
    {
        // We don't really care in this case, but good to know it loaded.
        NSLog(@"Loaded contents");
    }
    else
    {
        // Note that a given "basemap" webmap has loaded.
        NSString *loadedWebMapID = webMap.portalItem.itemId;
        [self.loadedStates setObject:[NSNumber numberWithBool:YES] 
                              forKey:loadedWebMapID];
    }
}

// Note any errors in the console log. You'd want to do more in real life
- (void) webMap:(AGSWebMap *)webMap didFailToLoadWithError:(NSError *)error
{
    NSLog(@"Error: %@", error);
}

// When a webmap opens into a mapview, this is called (again, delegates above)
- (void) didOpenWebMap:(AGSWebMap *)webMap intoMapView:(AGSMapView *)mapView
{
    // If self.mapExtentAtBasemapSwitch is nil, this is the initial load.
    if (self.mapExtentAtBasemapSwitch)
    {
        // Let's set the extent back to what it was before we switched basemaps...
        [self.mapView zoomToEnvelope:self.mapExtentAtBasemapSwitch animated:NO];
        self.mapExtentAtBasemapSwitch = nil;
    }
}

#pragma mark - Change the basemap

- (IBAction)basemapTypeChanged:(id)sender
{
    UISegmentedControl *sc = (UISegmentedControl *)sender;
    
    // Get hold of the basemap we want to switch to.
    AGSWebMap *newWebMapForBaseMap = nil;
    switch (sc.selectedSegmentIndex) {
        case kTopoIndex:
            newWebMapForBaseMap = self.topoWebMap;
            break;
        case kStreetIndex:
            newWebMapForBaseMap = self.streetWebMap;
            break;
        case kGreyIndex:
            newWebMapForBaseMap = self.greyWebMap;
            break;
        case kImageryIndex:
            newWebMapForBaseMap = self.imageryWebMap;
            break;
        default:
            NSLog(@"Oops - this shouldn't be!");
            break;
    }
    
    if (newWebMapForBaseMap)
    {
        // Given the webmap for the new basemap, now let's just check that
        // we got a loaded notification for this webmap.
        NSString *newBaseMapItemID = newWebMapForBaseMap.portalItem.itemId;
        NSNumber *isLoaded = [self.loadedStates objectForKey:newBaseMapItemID];
        
        if (isLoaded)
        {
            // Everything's good - we should be able to load up the basemap
            // from the selected basemap underneath the real "content" webmap.
            AGSWebMapBaseMap *newBaseMap = newWebMapForBaseMap.baseMap;
            self.mapExtentAtBasemapSwitch = self.mapView.visibleArea.envelope;
            [self.mainWebMapWithContents openIntoMapView:self.mapView
                                    withAlternateBaseMap:newBaseMap];
            return;
        }
    }
    
    // If we're still here, (i.e. the above "return" didn't kick us out)
    // then there's something wrong with the basemap we're trying to use.
    NSLog(@"Attempt to use a webmap which hasn't loaded yet!");
}

#pragma mark - Standard iOS stuff.

- (void)viewDidUnload
{
    [self setMapView:nil];
    
    self.mainWebMapWithContents = nil;
    
    self.topoWebMap = nil;
    self.streetWebMap = nil;
    self.imageryWebMap = nil;
    self.greyWebMap = nil;
    
    [super viewDidUnload];
    // Release any retained subviews of the main view.
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
}
@end
